<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Block;

use Flat101\EnhancedEcommerce\Api\Data\DataLayerInterface;
use Magento\Framework\View\Element\Template;
use Flat101\EnhancedEcommerce\Api\DataLayerListInterface;
use Flat101\EnhancedEcommerce\Factory\DataLayerFactory;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Event\Manager;
use Magento\Framework\Serialize\SerializerInterface;
use Magento\Framework\UrlInterface;
use Magento\Framework\View\Element\Template\Context;

abstract class AbstractDatalayer extends Template
{
    /** @var DataLayerFactory */
    protected $dataLayerFactory;

    /** @var DataLayerListInterface */
    protected $dataLayerList;

    /** @var Config */
    protected $config;

    /** @var SerializerInterface */
    protected $serializer;

    /** @var UrlInterface */
    protected $url;

    /** @var array */
    protected $codes = [];

    /** @var RedirectInterface */
    protected $redirect;

    /** @var Manager */
    protected $eventManage;

    public function __construct(
        Context $context,
        DataLayerListInterface $dataLayerList,
        DataLayerFactory $dataLayerFactory,
        SerializerInterface $serializer,
        UrlInterface $url,
        Config $config,
        RedirectInterface $redirect,
        Manager $eventManage,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->config = $config;
        $this->dataLayerFactory = $dataLayerFactory;
        $this->dataLayerList = $dataLayerList;
        $this->serializer = $serializer;
        $this->url = $url;
        $this->redirect = $redirect;
        $this->eventManage = $eventManage;
    }

    abstract public function getJsonDataLayers(): ?string;

    public function canImplementEnhancedEcommerce(): ?bool
    {
        return $this->config->canImplementEnhancedEcommerce($this->_storeManager->getStore()->getid());
    }

    public function getTypeDataLayers(): ?array
    {
        return $this->getData('layers');
    }

    public function getCodes(): array
    {
        return $this->codes;
    }

    /**
     * Get Instance of DataLayer
     */
    protected function getInstance(string $type): ?DataLayerInterface
    {
        $nameInstance = $this->findClass($type);
        if ($nameInstance && class_exists($nameInstance)) {
            return $this->dataLayerFactory->create($nameInstance);
        }

        return null;
    }

    protected function findClass(string $type): string
    {
        $list = $this->dataLayerList->getList();

        return $list[$type] ?? '';
    }

    public function getCurrentCurrency(): string
    {
        return $this->_storeManager->getStore()->getCurrentCurrency()->getCode();
    }
}
