<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Controller\Click;

use Flat101\EnhancedEcommerce\Factory\ClickImpressionHandlerFactory;
use Flat101\EnhancedEcommerce\Model\DataLayer\Impressions\ClickImpressionView;
use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Serialize\SerializerInterface;
use Psr\Log\LoggerInterface;
use Magento\Framework\App\RequestInterface;
use Magento\Framework\App\ResponseInterface;

class View implements ActionInterface
{
    /** @var RedirectInterface */
    protected $redirect;

    /** @var ClickImpressionHandlerFactory */
    protected $clickImpressionHandlerFactory;

    /** @var ProductRepositoryInterface */
    protected $productRepository;

    /** @var ClickImpressionView */
    protected $clickImpressionView;

    /** @var SerializerInterface */
    protected $serializer;

    /** @var LoggerInterface */
    protected $logger;

    /** @var RequestInterface */
    protected $request;

    /** @var ResponseInterface */
    protected $response;

    public function __construct(
        RedirectInterface $redirect,
        ClickImpressionHandlerFactory $clickImpressionHandlerFactory,
        ProductRepositoryInterface $productRepository,
        ClickImpressionView $clickImpressionView,
        SerializerInterface $serializer,
        LoggerInterface $logger,
        RequestInterface $request,
        ResponseInterface $response
    ) {
        $this->redirect = $redirect;
        $this->clickImpressionHandlerFactory = $clickImpressionHandlerFactory;
        $this->productRepository = $productRepository;
        $this->clickImpressionView = $clickImpressionView;
        $this->serializer = $serializer;
        $this->logger = $logger;
        $this->request = $request;
        $this->response = $response;
    }

    /**
     * {@inheritdoc}
     */
    public function execute()
    {
        if ($this->request->isAjax()) {
            $currentUrl = $this->request->getParam('currentUrl');
            $refererUrl = $this->request->getParam('refererUrl');

            if (null === $refererUrl || $refererUrl === $currentUrl) {
                return $this->response->representJson('');
            }

            $productId = (int) $this->request->getParam('id');
            $clickData = $this->serializer->unserialize($this->request->getParam('clickData'));
            if ($productId) {
                try {
                    $product = $this->productRepository->getById($productId);

                    $handlerImpressionClick = $this->clickImpressionHandlerFactory->create($refererUrl);
                    $handlerImpressionClick->handle($product, $clickData);

                    return $this->response->representJson(
                        $this->serializer->serialize($this->clickImpressionView->getLayer())
                    );
                } catch (\Exception $e) {
                    $this->logger->critical($e->getMessage());
                }
            }
        }

        return $this->response->representJson('');
    }
}
