<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Helper;

use Flat101\EnhancedEcommerce\Model\Config\Source\CategoryFormat;
use Flat101\EnhancedEcommerce\Model\Config\Source\VariantUse;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Store;
use Flat101\Analytics\Helper\Gtm As GtmHelper;

class Config extends AbstractHelper
{
    private const XML_PATH_PRODUCT_IDENTIFIER               = 'flat101_analytics/datalayers/product_identifier';
    private const XML_PATH_BRAND_ENABLE                     = 'flat101_analytics/datalayers/brand_enable';
    private const XML_PATH_BRAND_ATTRIBUTE                  = 'flat101_analytics/datalayers/brand_attribute';
    private const XML_PATH_CATEGORY_FORMAT                  = 'flat101_analytics/datalayers/category_format';
    private const XML_PATH_CATEGORY_SEPARATOR               = 'flat101_analytics/datalayers/category_separator';
    private const XML_PATH_DEFAULT_CATEGORY_SEPARATOR       = '/';
    private const XML_PATH_VARIANT_USE                      = 'flat101_analytics/datalayers/variant_use';
    private const XML_PATH_LIST_LABEL_CATALOG_IMPRESSIONS   = 'flat101_analytics/datalayers/catalog_impressions_list_label';
    private const XML_PATH_LIST_LABEL_SEARCH_IMPRESSIONS    = 'flat101_analytics/datalayers/search_impressions_list_label';
    private const XML_PATH_LIST_LABEL_RELATED_IMPRESSIONS   = 'flat101_analytics/datalayers/related_impressions_list_label';
    private const DEFAULT_LIST_LABEL_CATALOG_IMPRESSIONS    = 'Catalog List';
    private const DEFAULT_LIST_LABEL_SEARCH_IMPRESSIONS     = 'Search Results';
    private const DEFAULT_LIST_LABEL_RELATED_IMPRESSIONS    = 'CrossSelling - Related Products';

    /** @var GtmHelper */
    protected $gtmHelper;

    public function __construct(
        Context $context,
        GtmHelper $gtmHelper
    ) {
        parent::__construct($context);

        $this->gtmHelper = $gtmHelper;
    }

    /**
     * @param null $store_id
     * @return bool
     */
    public function canImplementEnhancedEcommerce($store_id = null): bool
    {
        return $this->gtmHelper->isActive($store_id);
    }

    public function getProductIdentifier($store_id = null): string
    {
        return $this->scopeConfig->getValue(
            self::XML_PATH_PRODUCT_IDENTIFIER,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );
    }

    public function isEnableBrand($store_id = null): bool
    {
        return $this->scopeConfig->isSetFlag(
            self::XML_PATH_BRAND_ENABLE,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );
    }

    public function getBrandAttribute($store_id = null): ?string
    {
        if ($this->isEnableBrand($store_id)) {
            return $this->scopeConfig->getValue(
                self::XML_PATH_BRAND_ATTRIBUTE,
                ScopeInterface::SCOPE_STORE,
                $store_id
            );
        }

        return null;
    }

    public function getCategoryFormat($store_id = null): int
    {
        $formatId = $this->scopeConfig->getValue(
            self::XML_PATH_CATEGORY_FORMAT,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );

        if (!is_null($formatId)) {
            return (int)$formatId;
        } else {
            return CategoryFormat::DEFAULT_FORMAT;
        }
    }

    public function getCategorySeparator($store_id = null): string
    {
        $separator = $this->scopeConfig->getValue(
            self::XML_PATH_CATEGORY_SEPARATOR,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );

        return $separator ?? self::XML_PATH_DEFAULT_CATEGORY_SEPARATOR;
    }

    public function getVariantUse($store_id = null): int
    {
        $variantUse = $this->scopeConfig->getValue(
            self::XML_PATH_VARIANT_USE,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );

        if (!is_null($variantUse)) {
            return (int)$variantUse;
        } else {
            return VariantUse::DEFAULT_USAGE;
        }
    }

    public function getListLabelCatalogImpressions($store_id = null): string
    {
        $catalogImpressions = $this->scopeConfig->getValue(
            self::XML_PATH_LIST_LABEL_CATALOG_IMPRESSIONS,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );

        return $catalogImpressions ?? self::DEFAULT_LIST_LABEL_CATALOG_IMPRESSIONS;
    }

    public function getListLabelSearchImpressions($store_id = null): string
    {
        $catalogImpressions = $this->scopeConfig->getValue(
            self::XML_PATH_LIST_LABEL_SEARCH_IMPRESSIONS,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );

        return $catalogImpressions ?? self::DEFAULT_LIST_LABEL_SEARCH_IMPRESSIONS;
    }

    public function getListLabelRelatedImpressions($store_id = null): string
    {
        $catalogImpressions = $this->scopeConfig->getValue(
            self::XML_PATH_LIST_LABEL_RELATED_IMPRESSIONS,
            ScopeInterface::SCOPE_STORE,
            $store_id
        );

        return $catalogImpressions ?? self::DEFAULT_LIST_LABEL_RELATED_IMPRESSIONS;
    }
}
