<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\DataLayer\Generator;

use Flat101\EnhancedEcommerce\Model\DataLayer\Builder\ImpressionBuilder;
use Flat101\EnhancedEcommerce\Model\DataLayer\Dto;
use Flat101\EnhancedEcommerce\Model\UnsetProperty;
use Magento\Catalog\Model\Product as ProductEntity;

class ClickImpression
{
    use UnsetProperty;

    public const EVENT = 'productClick';

    /**
     * @var ImpressionBuilder
     */
    private $impressionBuilder;

    public function __construct(ImpressionBuilder $impressionBuilder)
    {
        $this->impressionBuilder = $impressionBuilder;
    }

    public function generate(ProductEntity $entity, string $list, $additionalInfo = null): Dto\Ecommerce
    {
        $actionField = new Dto\Impression\ActionField();
        $actionField->list = $list;

        $detailsDto = new Dto\EcommerceDetails();
        $detailsDto->actionField = $actionField;
        $detailsDto->products[] = $this->impressionBuilder->build($entity, $list, $additionalInfo);

        $click = new Dto\Impression\ClickImpression();
        $click->click = $detailsDto;

        $ecommerce = new Dto\Ecommerce();
        $ecommerce->event = static::EVENT;
        $ecommerce->ecommerce = $click;

        $this->unset($ecommerce, ['eventCategory','eventAction','eventLabel']);

        return $ecommerce;
    }
}
