<?php
declare(strict_types=1);

namespace Flat101\EnhancedEcommerce\Model\Handler;

use Flat101\EnhancedEcommerce\Model\DataLayer\Generator\Impression;
use Flat101\EnhancedEcommerce\Model\Session;
use Flat101\EnhancedEcommerce\Helper\Config;
use Magento\Catalog\Model\ResourceModel\Product\Collection;
use Magento\Catalog\Block\Product\ProductList\Toolbar;

class SearchImpressionHandler implements ImpressionHandlerInterface
{
    /** @var Session */
    private $session;

    /** @var Impression */
    private $impressionGenerator;

    /** @var Toolbar */
    protected $toolbar;

    /** @var Config */
    protected $configHelper;

    public function __construct(
        Session $session,
        Impression $impressionGenerator,
        Toolbar $toolbar,
        Config $configHelper
    ) {
        $this->session = $session;
        $this->impressionGenerator = $impressionGenerator;
        $this->toolbar = $toolbar;
        $this->configHelper = $configHelper;
    }

    public function handle(Collection $collection): void
    {
        $this->session->getImpressionSearchProducts(true);

        $pageSize = $this->toolbar->getLimit();
        $pageNumber = $this->toolbar->getCurrentPage();
        $position = ($pageNumber - 1) * $pageSize;
        $listLabel = $this->configHelper->getListLabelSearchImpressions();

        /** @var \Magento\Catalog\Model\Product $product */
        foreach ($collection as $product) {
            $position++;
            $this->session->setImpressionSearchProducts(
                $this->impressionGenerator->generate($product, $listLabel, ['position' => $position])
            );
        }
    }
}
